#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <iostream>
#include <vector>

#include "RenderWindow.hpp"
#include "Entity.hpp"
#include "Utils.hpp"

int main(int argc, char* args[])
{
	if (SDL_Init(SDL_INIT_VIDEO) > 0)
	{
		std::cout << "HEY.. SDL_Init HAS FAILED. SDL_ERROR: " << SDL_GetError() << std::endl;
	}

	if (!(IMG_Init(IMG_INIT_PNG)))
		std::cout << "IMG_Init has failed. Error: " << SDL_GetError() << std::endl;

	RenderWindow window("Nanocraft", 1280, 720);
	int windowRefreshRate = window.getRefreshRate();

	std::cout << windowRefreshRate << std::endl;

	SDL_Texture* grassTexture = window.loadTexture("res/gfx/grassblock.png");
	// Entity entities[3] = {Entity(0, 0, grassTexture),
	// 					  Entity(30, 0, grassTexture),
	// 					  Entity(30, 30, grassTexture)};

	std::vector<Entity> entities = {Entity(Vector2f(0, 100), grassTexture),
						  			Entity(Vector2f(24, 100), grassTexture),
						  			Entity(Vector2f(48, 100), grassTexture),
						  			Entity(Vector2f(72, 100), grassTexture),
						  		    Entity(Vector2f(96, 100), grassTexture),
						  		    Entity(Vector2f(120, 100), grassTexture),
						  			Entity(Vector2f(144, 100), grassTexture),
						  			Entity(Vector2f(168, 100), grassTexture),
						  			Entity(Vector2f(192, 100), grassTexture),
						  		    Entity(Vector2f(216, 100), grassTexture)};

	bool gameRunning = true;

	SDL_Event event;

	const float timeStep = 0.01f;
	float accumulator = 0.0f;
	float currentTime = utils::hireTimeInSeconds();

	while (gameRunning)
	{
		int startTicks = SDL_GetTicks();

		float newTime = utils::hireTimeInSeconds();
		float frameTime = newTime - currentTime;

		currentTime = newTime;

		accumulator += frameTime;

		while (accumulator >= timeStep)
		{
			while (SDL_PollEvent(&event))
			{
				if (event.type == SDL_QUIT)
					gameRunning = false;
			}

			accumulator -= timeStep;

		}

		const float alpha = accumulator / timeStep;

		window.clear();

		for (Entity& e : entities)
		{
			window.render(e);
		}

		window.display();

		int frameTicks = SDL_GetTicks() - startTicks;

		if (frameTicks < 1000 / windowRefreshRate)
			SDL_Delay(1000 / windowRefreshRate - frameTicks);

	}

	window.cleanUp();
	SDL_Quit();

	return 0;
}